'use strict';

exports.__esModule = true;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = remotedevEnhancer;

var _jsan = require('jsan');

var _catchErrors = require('remotedev-utils/lib/catchErrors');

var _catchErrors2 = _interopRequireDefault(_catchErrors);

var _filters = require('remotedev-utils/lib/filters');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function sender(data, sendTo, headers, status) {
  if (status && status.started) status.started(data);
  try {
    var f = fetch(sendTo, {
      method: 'POST',
      headers: _extends({
        'content-type': 'application/json'
      }, headers),
      body: JSON.stringify(data)
    });
    if (status && (status.done || status.failed)) {
      f.then(function (response) {
        return response.json();
      }).then(function (r) {
        if (r && r.id) {
          if (status.done) status.done(r.id);
        } else {
          if (status.failed) status.failed(r.error);
        }
      }).catch(function (err) {
        if (status && status.failed) status.failed(err.message || err);
      });
    }
  } catch (err) {
    if (status && status.failed) status.failed(err.message || err);
    console.warn(err);
  }
}

function send(data, options) {
  if (!options.beforeSending) {
    options.sender(data, options.sendTo, options.headers, options.sendingStatus);
  } else {
    options.beforeSending(data, function (aData) {
      options.sender(aData || data, options.sendTo, options.headers, options.sendingStatus);
    });
  }
}

function prepare(data, options, action, error) {
  var preloadedState = options.preloadedState;
  if (typeof preloadedState !== 'undefined') {
    preloadedState = (0, _jsan.stringify)(options.preloadedState, options.stringifyReplacer);
  }
  if (!options.userAgent) {
    if (typeof window !== 'undefined' && window.navigator && window.navigator.userAgent) {
      options.userAgent = window.navigator.userAgent;
    } else {
      options.userAgent = 'non-browser';
    }
  }

  return {
    type: options.type,
    action: action,
    payload: data && (0, _jsan.stringify)(data, options.stringifyReplacer),
    preloadedState: preloadedState,
    title: options.title ? options.title : undefined,
    description: options.description,
    screenshot: options.screenshot,
    version: options.version,
    appId: options.appId,
    userAgent: options.userAgent,
    user: options.user,
    meta: options.version,
    error: error
  };
}

function add(data, options, state) {
  if (options.maxAge < options.data.length) {
    var sData = options.data.shift();
    if (sData) {
      if (options.withState) options.preloadedState = sData.state;else options.preloadedState = options.states.shift();
    }
  }
  if (!options.withState) options.states.push(state);
  options.data.push(data);
}

function preSend(action, store, options) {
  if ((0, _filters.isFiltered)(action, options.filters)) return;
  var data = void 0;
  var onlyState = options.onlyState;
  var withState = options.withState;
  var sendOn = options.sendOn;

  var state = store.getState();

  if (options.actionSanitizer) action = options.actionSanitizer(action);
  if (options.stateSanitizer) state = options.stateSanitizer(state);

  if (onlyState) {
    options.preloadedState = state;
  } else if (withState) {
    data = { state: state, action: action };
  } else {
    data = action;
  }
  if (options.every) {
    send(prepare(data, options), options);
  } else {
    if (!onlyState) add(data, options, state);
    if (typeof sendOn === 'string' && sendOn === action.type || (typeof sendOn === 'undefined' ? 'undefined' : _typeof(sendOn)) === 'object' && sendOn.indexOf(action.type) !== -1) {
      send(prepare(options.data, options, action.type), options);
    }
  }
}

function watchExceptions(store, options) {
  (0, _catchErrors2.default)(function (errAction) {
    var prevAction = void 0;
    if (options.data) {
      prevAction = options.data[options.data.length - 1];
      if (prevAction) {
        if (prevAction.action && prevAction.action.type) prevAction = prevAction.action;
        prevAction = prevAction.type;
      }
    }

    preSend(errAction, store, options);
    send(prepare(options.data, options, prevAction, errAction), options);
  });
}

function remotedevEnhancer(options) {
  if (!options || !options.sendTo && !options.sender) {
    throw new Error('Provide at least `sendTo` or `sender` option for remotedev enhancer.');
  }
  if (!options.sender) options.sender = sender;
  if (options.onlyState) {
    if (!options.type) options.type = 'STATE';
  } else if (options.every) {
    if (!options.type) options.type = 'ACTION';
  } else {
    if (!options.maxAge) options.maxAge = 30;
    if (!options.type) options.type = !options.withState ? 'ACTIONS' : 'STATES';
    options.data = [];
  }
  if (!options.withState && options.maxAge) options.states = [];
  if (options.actionsWhitelist) {
    options.filters = { whitelist: (0, _filters.arrToRegex)(options.actionsWhitelist) };
  } else if (options.actionsBlacklist) {
    options.filters = { blacklist: (0, _filters.arrToRegex)(options.actionsBlacklist) };
  }

  return function (createStore) {
    return function (reducer, preloadedState, enhancer) {
      var store = createStore(reducer, preloadedState, enhancer);
      if (options.sendOnError) watchExceptions(store, options);

      var dispatch = function dispatch(action) {
        if (typeof options.preloadedState === 'undefined' && options.data && options.data.length === 0) {
          options.preloadedState = store.getState();
        }
        var r = store.dispatch(action);
        preSend(action, store, options);
        return r;
      };

      return _extends({}, store, {
        dispatch: dispatch
      });
    };
  };
}